/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.generator;

import java.awt.image.BufferedImage;

import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.results.SResultImage;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.lib.logger.Logger;

public class ImageGenerator
{
	
	

	
	public static final void generateImage(final SSimul simul)
	{
		try{
			
			Logger.log(" ");
			Logger.beginLog("Création de l'image...");
			

			if( simul.getSpecs().getIndependentNormalisation() ){
				
				// normalisation differente pour chaque image
				for(int iImgNb=0; iImgNb<simul.getImagesCount(); iImgNb++){
					
					
					// on regarde s'il faut la generer la matrice
					if(!simul.needGenerateStage(iImgNb, SSimul.STAGE_NORMALISATION)){
						Logger.log("Image n°" + iImgNb + " ignorée.");
						Logger.log("");
						continue;
					}else Logger.beginLog("Image n°" + iImgNb + "...");
					
					// on recupere une image
					SResultImage res = simul.getResults().getImageAt(iImgNb);
					
					// reinitialise l'image
					Logger.beginLog("Initialisation de la matrice de l'image...");
					res.initForImage();
					
					
					// on recherche la valeur maximale
					double dblUseThisMax = simul.getResults().getImageAt(iImgNb).computeMaxTotalElectrons();
					normalize(simul.getSpecs().getImageAt(iImgNb), simul.getResults().getImageAt(iImgNb), dblUseThisMax);
					Logger.endLog(true);
				}

				
			}else{
				
				// meme normalisation pour chaque image : recherche le max de toute les images
				double dblMaxIntensity = 0.0;
				for(int iImgNb=0; iImgNb<simul.getImagesCount(); iImgNb++){
					double dblIntensity = simul.getResults().getImageAt(iImgNb).computeMaxTotalElectrons();
					if(dblIntensity > dblMaxIntensity) dblMaxIntensity = dblIntensity;
				}

				// normalise
				for(int iImgNb=0; iImgNb<simul.getImagesCount(); iImgNb++){
					
					// on recupere une image
					SResultImage res = simul.getResults().getImageAt(iImgNb);
					
					// reinitialise l'image
					Logger.beginLog("Initialisation de la matrice de l'image...");
					if( res.initForImage() ) Logger.log("creation de la matrice.");
					else Logger.log("reutilisation de la matrice.");
					
					Logger.beginLog("Image n°" + iImgNb + "...");
					normalize(simul.getSpecs().getImageAt(iImgNb), simul.getResults().getImageAt(iImgNb), dblMaxIntensity);
					Logger.endLog(true);
				}
				
			}
			
			Logger.endLog(true);
			return;
		}catch(Exception ex) {Logger.fireError(ImageGenerator.class.getName(), "generateNormalization()", ex);}
		Logger.endLog(false);
	}
	
	
	
	
	
	
	
	

	private static final void normalize(final SSpecImage imageSpec, final SResultImage imageRes, final double dblUseThisMax)
	{
		try{
			double dblMaxIntensity = dblUseThisMax;
			double dblFactor = -1.0;
			
			Logger.beginLog("Normalisation des données...");
	
			double dblLocalMin = dblMaxIntensity * imageSpec.getNormalisationBlack() / 1000.0;
			double dblLocalMax = dblMaxIntensity * imageSpec.getNormalisationWhite() / 1000.0;
			if(dblMaxIntensity > dblLocalMax) dblMaxIntensity = dblLocalMax;
			Logger.log("intensité maximale modifiée : " + dblMaxIntensity + ".");

			// verifie que la matrice ne soit pas vide
			if (dblMaxIntensity <= 0) {
				Logger.log("impossible de normaliser.");
				Logger.endLog(false);
				return;
			}

			// facteur mutiplicatif
			dblFactor = 255.0 / (dblLocalMax - dblLocalMin);
			imageRes.setNormalisationFactor(dblFactor);
			Logger.log("facteur de normalisation : " + dblFactor + ".");
			
			// normalise les pixels
			for(int i = 0; i < imageSpec.getImageWidth(); i++){
				for(int j = 0; j < imageSpec.getImageHeight(); j++){

					if(imageRes.getTotalElectrons(i, j) <= dblLocalMin){
						imageRes.setNormalizedValue(i, j, 0);
					}else{
						imageRes.setNormalizedValue(i, j, (int) Math.round( ( imageRes.getTotalElectrons(i, j) - dblLocalMin ) * dblFactor ) );
					}
					
				}
			}
			Logger.log("pixels normalisés.");
			
			

			// on cre une image vide
			BufferedImage img = imageRes.getNormalizedImage();
			Logger.log("taille de l'image : " + imageSpec.getImageWidth() + "px X " + imageSpec.getImageHeight() + "px.");

			// on copie les pixels uns a uns
			for(int i = 0; i < imageSpec.getImageWidth(); i++){
				for(int j = 0; j < imageSpec.getImageHeight(); j++){
					img.setRGB(i, j, imageRes.getRGBColorValue(i, imageSpec.getImageHeight()-1-j));
				}
			}
			
			Logger.log("image créée.");
			
			// ok

			Logger.endLog(true);
			return;
		}catch(Exception ex) {Logger.fireError(ImageGenerator.class.getName(), "normalize()", ex);}
		Logger.endLog(false);
	}
	
	
	
	

	

	public static final void generateColoratedImage(final SSimul simul)
	{
		try{
			Logger.beginLog("Création de l'image...");
			BufferedImage img = null;
			
			SResultImage imageRed = null;
			if(simul.getSpecs().getColoratedImageRed() > -1) imageRed = simul.getResults().getImageAt(simul.getSpecs().getColoratedImageRed());
			
			SResultImage imageGreen = null;
			if(simul.getSpecs().getColoratedImageGreen() > -1) imageGreen = simul.getResults().getImageAt(simul.getSpecs().getColoratedImageGreen());
			
			SResultImage imageBlue = null;
			if(simul.getSpecs().getColoratedImageBlue() > -1) imageBlue = simul.getResults().getImageAt(simul.getSpecs().getColoratedImageBlue());
			
			int iSizeX = 0;
			int iSizeY = 0;

			if( (imageRed != null) && (imageRed.getNormalizedImage() != null) ){
				iSizeX = imageRed.getImageWidth();
				iSizeY = imageRed.getImageHeight();
			}else imageRed = null;
			
			if( (imageGreen != null) && (imageGreen.getNormalizedImage() != null) ){
				if( iSizeX == 0 ){
					iSizeX = imageGreen.getImageWidth();
					iSizeY = imageGreen.getImageHeight();
				}else{
					if( (iSizeX != imageGreen.getImageWidth()) || (iSizeY != imageGreen.getImageHeight()) ){
						Logger.log("les tailles des images ne sont pas égales.");
						Logger.endLog(false);
						return;
					}
				}
			}else imageGreen = null;
			
			if( (imageBlue != null) && (imageBlue.getNormalizedImage() != null) ){
				if( iSizeX == 0 ){
					iSizeX = imageBlue.getImageWidth();
					iSizeY = imageBlue.getImageHeight();
				}else{
					if( (iSizeX != imageBlue.getImageWidth()) || (iSizeY != imageBlue.getImageHeight()) ){
						Logger.log("les tailles des images ne sont pas égales.");
						Logger.endLog(false);
						return;
					}
				}
			}else imageBlue = null;
			
			if( (iSizeX < 1) || (iSizeY < 1) ){
				Logger.log("aucune image couleur à créer.");
				Logger.endLog(false);
				return;
			}
			
			// on cre une image vide
			img = new BufferedImage(iSizeX, iSizeY, BufferedImage.TYPE_INT_RGB);
			Logger.log("taille de l'image couleur : " + iSizeX + "px X " + iSizeY + "px.");

			// on copie les pixels uns a uns
			int red = 0, green = 0, blue = 0;
			for(int i = 0; i < iSizeX; i++){
				for(int j = 0; j < iSizeY; j++){
					if( imageRed != null   ) red   = imageRed.getNormalizedValue(i, j);
					if( imageGreen != null ) green = imageGreen.getNormalizedValue(i, j);
					if( imageBlue != null  ) blue  = imageBlue.getNormalizedValue(i, j);
					img.setRGB(i, j, ((255 & 0xFF) << 24) | ((red & 0xFF) << 16) | ((green & 0xFF) << 8) | ((blue & 0xFF) << 0));
				}
			}
			
			simul.getResults().setColorImage(img);
			
			Logger.log("image couleur créée.");
			Logger.endLog(true);
			return;
		}catch (Exception ex) {Logger.fireError(ImageGenerator.class.getName(), "generateColoratedImage()", ex);}
		Logger.endLog(false);
	}

	

}

