/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.generator;

import sos.framework.maths.conversions.flux.FromJy;
import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.results.SResultImage;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.framework.simulation.simul.specs.sources.SSource;
import sos.lib.logger.Logger;

public class SourcesGenerator
{
	

	public static final void generateSources(final SSimul simul)
	{
		try{
			
			Logger.log(" ");
			Logger.beginLog("Génération des sources...");
			
			for(int iImgNb=0; iImgNb<simul.getImagesCount(); iImgNb++){
				
				// on regarde s'il faut la generer la matrice
				if(!simul.needGenerateStage(iImgNb, SSimul.STAGE_GENERATION)){
					Logger.log("Image n°" + iImgNb + " ignorée");
					Logger.log("");
					continue;
				}else Logger.beginLog("Image n°" + iImgNb + "...");
				
				// on recupere les images
				SSpecImage spec = simul.getSpecs().getImageAt(iImgNb);
				SResultImage res = simul.getResults().getImageAt(iImgNb);
				
				// on initialise la matrice
				Logger.beginLog("Initialisation de la matrice des sources...");
				if( res.initForSources(spec.getImageWidth(), spec.getImageHeight(), spec.getPixelResolution()) ) Logger.log("creation de la matrice.");
				else Logger.log("reutilisation de la matrice.");

				// optimisation
				int iSizeMax = spec.getGenerationObjectsSize();
				int iSizeMaxOverTwo = iSizeMax / 2;
				iSizeMax = 2*iSizeMaxOverTwo;
	
				// positions absolues de la matrice temporaire
				int xAbsStart;
				int yAbsStart;
	
				// matrice temporaire
				double[][] tmp = new double[iSizeMax][iSizeMax];
	
				// distances actuelles du centre
				double xDistFromCenter;
				double yDistFromCenter;
				
				// valeurs des flux
				double dblTotalElect = 0.0;
				double dblElect;
		
				// on genere les composants
				if((simul.getSpecs().getSourcesContainer() != null) && (simul.getSpecs().getSourcesContainer().getSourcesCount() > 0)){
					
					// on prend le premier composant
					SSource cpn = simul.getSpecs().getSourcesContainer().getSourceAt(0);
	
					/* on va calculer la distribution de la lumiere une seule fois
					 * car pour l'instant toutes les sources ont la meme forme,
					 * il faudra changer cela plus tard... 
					 */
					
					dblTotalElect = 0.0;
					for(int y = 0; y < iSizeMax; y++){
						for(int x = 0; x < iSizeMax; x++){
							
							// calcule la distance du centre de l'objet en arcsec
							//xDistFromCenter = ((int)cpn.getPosX()) - cpn.getPosX() - iSizeMax/2 + x + 0.5;
							//yDistFromCenter = ((int)cpn.getPosY()) - cpn.getPosY() - iSizeMax/2 + y + 0.5;
							xDistFromCenter = ( x + 0.5 - iSizeMaxOverTwo ) * spec.getPixelResolution();
							yDistFromCenter = ( y + 0.5 - iSizeMaxOverTwo ) * spec.getPixelResolution();
							
							// calcule l'intensite arbitraire, pour un pixel carre
							dblElect = cpn.calculateIntensity(xDistFromCenter, yDistFromCenter, spec);
							
							// convertie en nombre d'electron
							dblElect = FromJy.toElec(dblElect, spec);
							
							// memorise cette donnee dans la matrice
							dblTotalElect += dblElect;
							tmp[x][y] = dblElect;
						}
					}
					
					/* on connais maintenant la distribution de la lumiere, on
					 * va garder cette distribution pour chaque objet, mais on
					 * appliquera a chacun un coefficient afin d'obtenir le flux
					 * total vraiment voulu (meme distribution mais pas meme
					 * intensité...)
					 */
					if(dblTotalElect > 0) {
						int j = 0;
						for(int i=0; i<simul.getSpecs().getSourcesContainer().getSourcesCount(); i++){
							
							cpn = simul.getSpecs().getSourcesContainer().getSourceAt(i);
		
							// positions absolues de la matrice temporaire
							xAbsStart = (int) ( spec.getImageWidth() / 2.0 + cpn.getPosX() / spec.getPixelResolution() - iSizeMaxOverTwo );
							yAbsStart = (int) ( spec.getImageHeight() / 2.0 + cpn.getPosY() / spec.getPixelResolution() - iSizeMaxOverTwo );
	
							// on calcule le facteur
							double dblFactor = FromJy.toElec(cpn.getTotalFlux(), spec) / dblTotalElect;
							
							// on stocke cette donnee dans la matrice principale
							for(int y = 0; y < iSizeMax ; y++){
								for(int x = 0; x < iSizeMax ; x++){
									// on verifie que l'on reste bien dans la matrice principale
									if( (xAbsStart + x > -1) && (xAbsStart + x < spec.getImageWidth()) && (yAbsStart + y > -1) && (yAbsStart + y < spec.getImageHeight())) {
										// flux reel
										dblElect = tmp[x][y] * dblFactor;
										// on ajoute le flux que l'on convertie de Jy en nombre d'electron
										res.addSourcesElectrons(xAbsStart + x, yAbsStart + y, dblElect);
									}
								}
								
							}
							j++;
							// met a jour que tout les 100 objets
							if(j > 100) {j = 0; /*simul.fireSimulationComponentsGenerated(i);*/}
						}
					}
				}

				
				Logger.endLog(true);
			}
			
			
			Logger.endLog(true);
			return;
		}catch(Exception ex) {Logger.fireError(SourcesGenerator.class.getName(), "generateObjects()", ex);}
		Logger.endLog(false);
	}

	
	
	

}

