/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.io;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.net.URLEncoder;
import java.util.Enumeration;

import sos.framework.maths.Maths;
import sos.framework.simulation.SEnv;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.framework.simulation.simul.specs.SSpecs;
import sos.framework.simulation.simul.specs.distrib.DistribFunction;
import sos.framework.simulation.simul.specs.filters.SFilter;
import sos.framework.simulation.simul.specs.noises.SNoise;
import sos.framework.simulation.simul.specs.sources.SSource;
import sos.lib.logger.Logger;

public class ExportSOS
{
	
	

	public static final void export(final SEnv env)
	{
		try{
			// fichier de destination
			File file = new File( env.getPrefs().getExportFile() );
			file.delete();
	 		BufferedWriter w = new BufferedWriter(new FileWriter(file));

	 		// les donnees
	 		SSpecs specs = env.getSimul().getSpecs();
	 		StringBuffer sb = new StringBuffer(10240);
	 		sb.append( "[SSPECS]\n" );
	 		sb.append( "VERSION  1\n" );

			if( env.getPrefs().getExportSpecs() ){
				sb.append( "IMAGE    " + env.getSimul().getActualImageNumber() + "\n" );
				sb.append( "DESC     " + ( env.getSimul().getSimulationDescription().length() == 0 ? "[NULL]" : URLEncoder.encode( env.getSimul().getSimulationDescription(), "UTF8") ) + "\n" );
				sb.append( "\n" );
				sb.append( saveNoises(specs) );
				sb.append( saveFilters(specs) );
				sb.append( saveVariables(specs) );
				sb.append( saveDistributionFunctions(specs) );
				sb.append( saveImages(specs) );
				sb.append( saveDistribution(specs) );
				sb.append( saveColoration(specs) );
			}else{
				sb.append( "IMAGE    -1\n" );
				sb.append( "\n" );
			}
		
			if( env.getPrefs().getExportSources() ){
				sb.append( saveSources(specs) );
			}
			
			String s = sb.toString();
	 		if(s == null){
	 			Logger.notifyInfo(env.getWindow(), "Erreur d'enregistrement : export=null", "Enregistrer l'image au format SOS");	 			
	 		}else{
		 		w.write(s);
		 		w.flush();
		 		w.close();
		 		Logger.notifyInfo(env.getWindow(), "Enregistrement terminé.", "Enregistrer l'image au format SOS");	 			
	 		}
	 		
		}catch(Exception ex){
			Logger.fireError(ExportSOS.class.getName(), "exportSOS()", ex);
			Logger.notifyError(env.getWindow(), "Erreur d'enregistrement : " + ex.getMessage(), "Enregistrer l'image au format SOS");
		}
	}
	
	

	
	
	
	
	
	
	

	
	private static final String saveImages(final SSpecs specs) throws Exception
	{
		String out = "";
		String ret;
		SSpecImage image;
		for(int i=0; i<specs.getImages().size(); i++){
			image = specs.getImageAt(i);
			ret = saveImage(specs, image);
			if(ret != null) out += ret;
			else return null;
		}
		return out;
	}

	private static final String saveImage(final SSpecs specs, final SSpecImage image)
	{
		String out = "[IMAGE]\n";
		out += "SIZEX    " + image.getImageWidth() + "\n";
		out += "SIZEY    " + image.getImageHeight() + "\n";
		out += "PIXRES   " + image.getPixelResolution() + "\n";
		out += "GENSIZE  " + image.getGenerationObjectsSize() + "\n";
		out += "NRMWHITE " + image.getNormalisationWhite() + "\n";
		out += "NRMBLACK " + image.getNormalisationBlack() + "\n";
		
		// filtre
		out += "FILTER   " + specs.getFilters().indexOf(image.getFilter()) + "\n";
		
		// variables
		for(Enumeration e=specs.getVariableNames(); e.hasMoreElements(); ){
			out += "VARIABLE " + image.getVariableValue((String)e.nextElement()) + "\n";
		}
		
		// bruits
		out += "NOISES   " + image.getNoises().size() + "\n";
		for(int i=0; i<image.getNoises().size(); i++){
			out += "NOISE    " + specs.getNoises().indexOf(image.getNoises().get(i)) + "\n";
		}
		
		return out + "\n";
	}

	private static final String saveVariables(final SSpecs specs) throws Exception
	{
		String out = "";
		for(Enumeration e = specs.getVariableNames(); e.hasMoreElements(); ){
			String varname = (String)(e.nextElement());
			out += "[VARIABLE]\n";
			out += "NAME     " + URLEncoder.encode(varname, "UTF8") + "\n";
			out += "DESC     " + URLEncoder.encode(specs.getVariableDescription(varname), "UTF8") + "\n";
			out += "\n";
		}
		return out;
	}
	
	private static final String saveFilters(final SSpecs specs) throws Exception
	{
		String out = "";
		for(int i=0; i<specs.getFilters().size(); i++){
			SFilter filter = (SFilter)(specs.getFilters().get(i));
			out += "[FILTER]\n";
			out += "DESC     " + URLEncoder.encode(filter.getDescription(), "UTF8") + "\n";
			out += "CENTER   " + filter.getLambdaCenter() + "\n";
			out += "DELTA    " + filter.getLambdaDelta() + "\n";
			out += "\n";
		}
		return out;
	}
	
	private static final String saveNoises(final SSpecs specs) throws Exception
	{
		String out = "";
		for(int i=0; i<specs.getNoises().size(); i++){
			SNoise noise = (SNoise)(specs.getNoises().get(i));
			out += "[NOISE]\n";
			out += "DESC     " + URLEncoder.encode(noise.getDescription(), "UTF8") + "\n";
			out += "FOREACH  " + noise.getApplyForEachPixel() + "\n";
			out += "FUNCTION " + URLEncoder.encode(noise.getFunction(), "UTF8") + "\n";
			out += "\n";
		}
		return out;
	}
	
	private static final String saveDistributionFunctions(final SSpecs specs) throws Exception
	{
		String out = "";
		for(int i=0; i<specs.getDistributionFunctions().size(); i++){
			DistribFunction function = (DistribFunction)(specs.getDistributionFunctions().get(i));
			out += "[FUNCTION]\n";
			out += "NAME     " + URLEncoder.encode(function.getName(), "UTF8") + "\n";
			//out += "COLOR    " + ( function.getColor() == null ? -1 : function.getColor().getRGB() ) + "\n";
			out += "FUNCTION " + URLEncoder.encode(function.getPoints(), "UTF8") + "\n";
			out += "\n";
		}
		return out;
	}
	
	private static final String saveDistribution(final SSpecs specs) throws Exception
	{
		String out = "";
		out += "[DISTRIBUTION]\n";
		out += "FUNCTION " + specs.getDistributionFunctions().indexOf(specs.getDistributionFunction()) + "\n";
		out += "GENERATE " + specs.isUseGenerator() + "\n";
		return out + "\n";
	}
	
	private static final String saveColoration(final SSpecs specs) throws Exception
	{
		String out = "[COLORATION]\n";
		out += "RED      " + specs.getColoratedImageRed() + "\n";
		out += "GREEN    " + specs.getColoratedImageGreen() + "\n";
		out += "BLUE     " + specs.getColoratedImageBlue() + "\n";
		return out + "\n";
	}
	
	private static final String saveSources(final SSpecs specs) throws Exception
	{
		StringBuffer sb = new StringBuffer( "[SOURCES]\n" );
		sb.append( "ORIGIN   " + specs.getSourcesContainer().getSourcesOrigin() + "\n" );
		sb.append( "NUMBER   " + specs.getSourcesContainer().getSourcesCount() + "\n" );
		
		for(int i=0; i<specs.getSourcesContainer().getSourcesCount(); i++){
			SSource source = specs.getSourcesContainer().getSourceAt(i);
			sb.append( "[" + Maths.toString( source.getPosX(), 3 ) + ";" + Maths.toString( source.getPosY(), 3 ) + ";" + Maths.toString( source.getTotalFlux(), 6 ) + "]\n" );
		}

		sb.append( "\n" );
		return sb.toString();
	}
	
	
	
	
	
	

}
