/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.maths;

import sos.framework.simulation.simul.SSimul;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.lib.logger.Logger;
import sos.lib.mathparser.ParserFactory;
import sos.lib.mathparser.functions.conversions.ConvElecPixToJySterad;
import sos.lib.mathparser.functions.conversions.ConvJySteradToElecPix;
import sos.lib.mathparser.functions.random.RandomGauss;
import sos.lib.mathparser.functions.random.RandomUniform;
import sos.lib.mathparser.functions.special.FuncGauss;
import sos.lib.mathparser.functions.special.FuncJ0;
import sos.lib.mathparser.functions.special.FuncJ1;
import sos.lib.mathparser.functions.special.FuncJn;
import sos.lib.mathparser.functions.special.FuncPlanck;
import sos.lib.mathparser.functions.special.FuncPoisson;
import sos.lib.mathparser.functions.special.FuncY0;
import sos.lib.mathparser.functions.special.FuncY1;
import sos.lib.mathparser.functions.special.FuncYn;


public class SOSParser
{
	
	// variables disponibles
	public static final String VAR_SOURCES_ELECT		= "sources_elec";
	public static final String VAR_MAX_SOURCES_ELECT	= "max_sources_elec";
	public static final String VAR_TOTAL_ELECT			= "total_elec";
	public static final String VAR_MAX_TOTAL_ELECT		= "max_total_elec";
	public static final String VAR_X					= "x";
	public static final String VAR_Y					= "y";
	
	public static final String VAR_OBS_TIME				= "obs_time";
	public static final String VAR_LAMBDA				= "lambda";
	public static final String VAR_DLAMBDA				= "dlambda";
	public static final String VAR_TELESC_DIAM			= "telesc_diam";
	public static final String VAR_PIX_RES				= "pix_res";

	
	
	
	
	
	
	private static ParserFactory functionsParser		= null;

	public static final ParserFactory getFunctionsParser()
	{
		try{
			if(functionsParser != null) return functionsParser;
			functionsParser = new ParserFactory();
		}catch(Exception ex) {Logger.fireError(SOSParser.class.getName(), "getFunctionsParser()", ex);}
		return functionsParser;
	}
	
	
	
	
	
	
	

	private static ParserFactory noisesParser			= null;
	
	
	public static final ParserFactory getNoisesParser()
	{
		try{
			if(noisesParser != null) return noisesParser;
			createNoisesParserVariables();
		}catch(Exception ex) {Logger.fireError(SOSParser.class.getName(), "getNoisesParser()", ex);}
		return noisesParser;
	}
	
	private static final void createNoisesParserVariables()
	{
		try{
			noisesParser = new ParserFactory();
			
			noisesParser.addFunction(new FuncPlanck("planck"));
			noisesParser.addFunction(new FuncGauss("gauss"));
			noisesParser.addFunction(new FuncPoisson("poisson"));
			
			noisesParser.addFunction(new RandomUniform("randuni"));
			noisesParser.addFunction(new RandomGauss("randgauss"));
			
			noisesParser.addFunction(new FuncJ0("j0"));
			noisesParser.addFunction(new FuncJ1("j1"));
			noisesParser.addFunction(new FuncJn("jn"));
			noisesParser.addFunction(new FuncY0("y0"));
			noisesParser.addFunction(new FuncY1("y1"));
			noisesParser.addFunction(new FuncYn("yn"));
			
			noisesParser.setValue("c", Constants.C);
			noisesParser.setValue("e", Constants.E);
			noisesParser.setValue("h", Constants.H);
			noisesParser.setValue("k", Constants.K);
			noisesParser.setValue("q", Constants.Q);
			noisesParser.setValue("pi", Constants.PI);
			noisesParser.setValue("NaN", Double.NaN);
			
		}catch(Exception ex) {Logger.fireError(SOSParser.class.getName(), "getNoisesParser()", ex);}
	}
	
	
	
	public static final void updateNoisesParserVariablesForImage(final SSpecImage imageSpec)
	{
		if(noisesParser == null) createNoisesParserVariables();
		
		noisesParser.setValue(VAR_OBS_TIME, imageSpec.getVariableValue(SSimul.VAR_OBSERVATION_TIME_NAME));
		noisesParser.setValue(VAR_TELESC_DIAM, imageSpec.getVariableValue(SSimul.VAR_TELESCOPE_DIAMETER_NAME));
		noisesParser.setValue(VAR_PIX_RES, imageSpec.getPixelResolution());
		noisesParser.setValue(VAR_LAMBDA, imageSpec.getFilter().getLambdaCenter());
		noisesParser.setValue(VAR_DLAMBDA, imageSpec.getFilter().getLambdaDelta());
		
		noisesParser.setValue(VAR_SOURCES_ELECT, Double.NaN);
		noisesParser.setValue(VAR_MAX_SOURCES_ELECT, Double.NaN);
		noisesParser.setValue(VAR_TOTAL_ELECT, Double.NaN);
		noisesParser.setValue(VAR_MAX_TOTAL_ELECT, Double.NaN);
		noisesParser.setValue(VAR_X, Double.NaN);
		noisesParser.setValue(VAR_Y, Double.NaN);
		
		noisesParser.addFunction(new ConvElecPixToJySterad("elect.pix}jy.sterad", imageSpec));
		noisesParser.addFunction(new ConvJySteradToElecPix("jy.sterad}elect.pix", imageSpec));
	}
	
	
	
	
	
}
