/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.simulation.simul;

import java.util.Vector;

import javax.swing.JOptionPane;

import sos.framework.io.SOSInputOutput;
import sos.framework.simulation.simul.results.SResultImage;
import sos.framework.simulation.simul.results.SResults;
import sos.framework.simulation.simul.specs.SSpecImage;
import sos.framework.simulation.simul.specs.SSpecs;
import sos.framework.ui.SOSFrame;
import sos.lib.logger.Logger;

public class SSimul
{

	private static final long serialVersionUID 				= 1L;

	public static final int STAGE_INITIALISATION			= 0;
	public static final int STAGE_DISTRIBUTION				= 1;
	public static final int STAGE_GENERATION				= 2;
	public static final int STAGE_NOISES					= 3;
	public static final int STAGE_NORMALISATION				= 4;
	public static final int STAGE_END						= 5;
	
	public static final int STAGES_NUMBER					= 6;
	
	public static final int DISTRIBUTION_FILE				= -10;
	public static final int DISTRIBUTION_SCRIPT				= -11;

	public static final String VAR_OBSERVATION_TIME_NAME	= "observation_time__s";
	public static final String VAR_TELESCOPE_DIAMETER_NAME	= "telescope_diameter__m";

	



	
	public SSimul() {}


	
	
	
	public final boolean startSimulation(final SOSFrame window, final String strFileName)
	{
		try{
			// cree les specs et les resulats
			this.specs = new SSpecs(this);
			this.results = new SResults(this);

			// cree une nouvelle simulation
			if( SOSInputOutput.importSOS(window, this, strFileName) == false ){
				endSimulation();
				return false;
			}
	
			// selectionne une image
			if(this.getImagesCount() == 3) setActualImageNumber(1);
			else setActualImageNumber(0);
			
			return true;
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "startSimulation()", ex);}
		return false;
	}

	
	public final void endSimulation()
	{
		try{
			// supprime les specs
			if( specs != null ) this.specs.dispose();
			this.specs = null;
			
			// supprime les resultats
			if( results != null) this.results.dispose();
			this.results = null;
			
			// supprime les etats
			lstNeedStates.clear();
			
			// plus d'image actuelle
			iActualImage = -1;
			
			// nottifie que la simulation n'existe plus
			fireSimulationImageChanged();

			// nettoie la memoire
			System.gc();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "endSimulation()", ex);}
	}
	
	
	
	
	

	

	
	private SSpecs specs									= null;
	public final SSpecs getSpecs() {return specs;}
	
	private SResults results								= null;
	public final SResults getResults() {return results;}
	

	
	
	
	
	
   	private String strSimulationDescription					= "";
   	public final String getSimulationDescription() {return strSimulationDescription;}
   	public final void setSimulationDescription(final String strSimulationDescription) {this.strSimulationDescription = strSimulationDescription;}

   	

   	
   	
	
	
	// index de l'image actuelle
	private int iActualImage								= -1;
	
	public final int getActualImageNumber() {return iActualImage;}

	public final void setActualImageNumber(final int newImageNumber)
	{
		if(specs == null) return;
		try{
			if( (newImageNumber > -1) && (newImageNumber < specs.getImages().size()) ){
				iActualImage = newImageNumber;
				fireSimulationImageChanged();
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "setActualImageNumber()", ex);}
	}
	
	public final int getImagesCount()
	{
		if(specs == null) return 0;
		return specs.getImages().size();
	}
	

	
	
	
	
	

	// duplique une image
	public final void duplicateActualImage()
	{
		try{
			// image spec
			SSpecImage newImage = specs.getActualImage().getCloneImage();
			addImage(newImage, this.needGenerateStage(this.getActualImageNumber(), STAGE_DISTRIBUTION));
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "duplicateImage()", ex);}
	}
	
	// ajoute une image
	public final void addImage(final SSpecImage newImage) {addImage(newImage, true);}
	public final void addImage(final SSpecImage newImage, final boolean doDistribution)
	{
		try{
			// image spec
			specs.getImages().add(newImage);
			
			// image res
			SResultImage newResults = new SResultImage();
			newResults.initForSources(newImage.getImageWidth(), newImage.getImageHeight(), newImage.getPixelResolution());
			newResults.initForNoises(newImage.getNoises());
			newResults.initForImage();
			results.getImages().add(newResults);
			
			// image stats
			lstNeedStates.add(new boolean[] {doDistribution, true, true, true});
			
			this.updateImageNumbers();
			this.setActualImageNumber(specs.getImages().size()-1);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "addImage()", ex);}
	}
	
	// supprime une image
	public final void deleteActualImage(final SOSFrame window)
	{
		try{
			if(specs.getImages().size() < 2) return;
			int ret = JOptionPane.showConfirmDialog(window, "Etes-vous sur de vouloir supprimer définitivement cette image ?", "Confirmation ?", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
		
			if( ret == JOptionPane.OK_OPTION ){
				
				// image spec
				specs.getImageAt(this.getActualImageNumber()).destroyImage();
				specs.getImages().remove(iActualImage);
				
				// image res
				results.getImageAt(this.getActualImageNumber()).destroyImage();
				results.getImages().remove(iActualImage);
				
				// image stats
				lstNeedStates.remove(iActualImage);
				
				this.updateImageNumbers();
				if(iActualImage == 0) this.setActualImageNumber(0);
				else this.setActualImageNumber(iActualImage-1);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "deleteActualImage()", ex);}
	}
	
	public final void updateImageNumbers()
	{
		for(int i=0; i<this.getImagesCount(); i++) specs.getImageAt(i).setImageNumber(i);
	}
	

	

	
	

// ETATS DES IMAGES //
			

	private Vector lstNeedStates								= new Vector();

	public final boolean needGenerateStage(final int iImageNumber, final int iStageNumber)
	{
		try{
			boolean[] tablNeededStages = (boolean[])(lstNeedStates.get(iImageNumber));
			return tablNeededStages[iStageNumber-1];
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "needGenerateStage()", ex);}
		return true;
	}
	
	public final void fireImageSpecsChanged(final int iImageNumber, final int iStageNumber)
	{
		try{
			// ignore si on est entrain de construire une simulation
			if( iImageNumber < 0 ) return;
			boolean blNeed = false;
			boolean[] tablNeededStages = (boolean[])(lstNeedStates.get(iImageNumber));
			// on memorise que toutes les etapes a partir de celle-ci, uniquement pour cette image, sont a refaire
			for(int i=iStageNumber-1; i<tablNeededStages.length; i++){
				if(tablNeededStages[i] != true){
					tablNeededStages[i] = true;
					blNeed = true;
				}
			}
			// on nottifie que si on a modifie l'image actuelle
			if((blNeed) && (iImageNumber == iActualImage)) fireSpecsChanged();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fireImageSpecsChanged()", ex);}
	}
	

	public final void fireResultsComputed(final int iStageNumber)
	{
		try{
			boolean blNeed = false;
			boolean[] tablNeededStages;
			// on memorise que cette etape est simulee, et ce pour toutes les images
			for(int i=0; i<this.getImagesCount(); i++){
				tablNeededStages = (boolean[])(lstNeedStates.get(i));
				if(tablNeededStages[iStageNumber-1] != false){
					tablNeededStages[iStageNumber-1] = false;
					if(i == iActualImage) blNeed = true;
				}
			}
			// on nottifie que si on a modifie l'image actuelle
			if(blNeed) fireSpecsChanged();
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "fireResultsComputed()", ex);}
	}
	
		
		
			
		
		
		
	
	
	
	
	
// ETAT DE LA SIMULATION //

		
	// memorise si une simulation est en cours
	private boolean blIsSimulating 							= false;
	
	public final boolean isSimulating() {return blIsSimulating;}
	
	public final void setSimulating(boolean blIsSimulating)
	{
		if( this.blIsSimulating != blIsSimulating ){
			this.blIsSimulating = blIsSimulating;
			fireSimulationStateChanged(blIsSimulating);
		}
	}
	
	// memorise l'etat de la simulation en cours
	private int iSimulationStage							= STAGE_INITIALISATION;
	
	public final int getSimulationStage() {return iSimulationStage;}
	
	public final void setSimulationStage(int iSimulationStage)
	{
		if(this.iSimulationStage != iSimulationStage){
			// change l'etat
			this.iSimulationStage = iSimulationStage;
			fireSimulationStageChanged(iSimulationStage);
		}
	}
	

	
	
	
// LISTENERS //
	

	private Vector lstSimulListeners = new Vector();
	public final void addSimulationListener(final _SSpecsListener listener) {lstSimulListeners.add(listener);}
	public final void removeSimulationListener(final _SSpecsListener listener) {lstSimulListeners.remove(listener);}

	public final void fireSimulationStageChanged(final int iSimulationStage) {for(int i=0; i<lstSimulListeners.size(); i++) try{ ((_SSpecsListener)(lstSimulListeners.get(i))).onSimulationStageChanged(iSimulationStage); }catch(Exception ex) {}}
	public final void fireSimulationStateChanged(final boolean blIsSimulating) {for(int i=0; i<lstSimulListeners.size(); i++) try{ ((_SSpecsListener)(lstSimulListeners.get(i))).onSimulationStateChanged(blIsSimulating); }catch(Exception ex) {}}
	public final void fireSimulationImageChanged() {for(int i=0; i<lstSimulListeners.size(); i++) try{ ((_SSpecsListener)(lstSimulListeners.get(i))).onSimulationImageChanged(); }catch(Exception ex) {}}
	


	
	
	private Vector lstSpecsListeners = new Vector();
	public final void addSpecsListener(final _SImageListener listener) {lstSpecsListeners.add(listener);}
	public final void removeSpecsListener(final _SImageListener listener) {lstSpecsListeners.remove(listener);}
	
	private final void fireSpecsChanged() {for(int i=0; i<lstSpecsListeners.size(); i++) try{ ((_SImageListener)(lstSpecsListeners.get(i))).onSpecsChanged();}catch(Exception ex) {}}
	

		
	
	
	private Vector lstSourcesListeners = new Vector();
	public final void addSourcesListener(final _SSourcesListener listener) {lstSourcesListeners.add(listener);}
	public final void removeSourcesListener(final _SSourcesListener listener) {lstSourcesListeners.remove(listener);}

	public final void fireSimulationComponentsChanged() {for(int i=0; i<lstSourcesListeners.size(); i++) try{ ((_SSourcesListener)(lstSourcesListeners.get(i))).onSimulationComponentsChanged(); }catch(Exception ex) {}}
	public final void fireSimulationComponentsGenerated(int iGeneratedNumber) {for(int i=0; i<lstSourcesListeners.size(); i++) try{ ((_SSourcesListener)(lstSourcesListeners.get(i))).onSimulationComponentsGenerated(iGeneratedNumber); }catch(Exception ex) {}}
	

	


}

