/*
 SOS Project Classes.

 Copyright (C) SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package sos.framework.simulation.simul.results;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.util.Vector;

import sos.framework.simulation.simul.specs.noises.SSpecNoises;
import sos.lib.logger.Logger;

/**
 * 
 * @author Camille AMBROSI
 * 
 * Une matrice de pixel contient toutes les donnees de l'image. La valeur de
 * chaque pixel est le nombre d'electron par seconde par pixel
 * [elec.s(-1).px(-1)]. Cette valeur peut facilement etre convertie en jansky
 * par steradian [Jy.sterad(-1)] grace a la formule suivante : <code>
 *   flux [Jy.px(-1)] 		= flux [elec.s(-1).px(-1)] * 
 *   flux [Jy.arcsec(-1)]	= flux [Jy.px(-1)] / pixres [arcsec.px[-1]]
 *   flux [Jy.sterad(-1)]	= flux [Jy.arcsec(-1)] * PI / 3600^2 / 180^2
 * </code>
 * 
 */
public class SResultImage
{

	// la matrice des sources
	private double[][] sources					= null;
	
	// la matrice des bruits totaux
	private double[][] noises					= null;
	
	// les matrices des bruits
	private Vector lstNoises					= new Vector();
	
	// la matrice normalisee
	private byte[][] norma						= null;

	// l'image
	private BufferedImage image					= null;
	


	
	
	public final void destroyImage()
	{
		try{
			sources = null;
			sources = null;
			norma = null;
			image = null;
			lstNoises.clear(); lstNoises = null;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "destroyImage()", ex);}
	}
	
	
	public boolean initForSources(final int x, final int y, final double pixRes)
	{
		this.pixelResolution = pixRes;
		
		// on regarde si on doit creer une nouvelle matrice ou juste la vider
		if( (sources == null) || (sources.length != x) || (sources[0].length != y) ){
			// on construit les nouvelles matrices
			sources = new double[x][y];
			return true;
		}else{
			for(int i = 0; i < x; i++){
				for(int j = 0; j < y; j++){
					sources[i][j] = 0.0;
				}
			}
			return false;			
		}
	}

	public boolean initForNoises(final SSpecNoises newNoisesList)
	{
		int x = sources.length;
		int y = sources[0].length;
		
		// regarde si on doit ajouter ou enlever des bruits
		while(lstNoises.size() > newNoisesList.size()) lstNoises.remove(0);
		while(newNoisesList.size() > lstNoises.size()) lstNoises.add(new SResultNoise());

		// initialise les bruits
		for(int i=0; i<newNoisesList.size(); i++){
			getGeneratedNoiseAt(i).initNoise(x, y, newNoisesList.get(i).getDescription());
		}
		
		// on regarde si on doit creer une nouvelle matrice ou juste la vider
		if( (noises == null) || (noises.length != x) || (noises[0].length != y) ){
			// on construit les nouvelles matrices
			noises = new double[x][y];
			return true;
		}else{
			for(int i = 0; i < x; i++){
				for(int j = 0; j < y; j++){
					noises[i][j] = 0.0;
				}
			}
			return false;	
		}
	}

	
	public boolean initForImage()
	{
		int x = sources.length;
		int y = sources[0].length;
		boolean newAlloc = false;

		// on regarde si on doit creer une nouvelle image ou juste la vider
		if( (norma == null) || (x != norma.length) || (y != norma[0].length) ){
	
			// nouvelle image
			norma = new byte[x][y];
			image = new BufferedImage(x, y, BufferedImage.TYPE_INT_RGB);
 			newAlloc = true;
		}

		// initialisation de l'ancienne image
		for(int i = 0; i < x; i++){
			for(int j = 0; j < y; j++){
				norma[i][j] = -128;
			}
		}
		
		Graphics g = image.getGraphics();
		g.setColor(Color.blue);
		g.fillRect(0, 0, x, y);
		
		return newAlloc;
	}
	
	
	
	
	

	
	

	
	public final void addSourcesElectrons(final int x, final int y, final double delta)
	{
		sources[x][y] += delta;
	}

	public final double getSourcesElectrons(final int x, final int y)
	{
		return sources[x][y];
	}
	
	public final double computeMaxSourcesElectrons()
	{
		try{
			double value = 0.0;
			double dblMaxIntensity = 0.0;
			for(int i = 0; i < sources.length; i++){
				for(int j = 0; j < sources[0].length; j++) {
					value = getSourcesElectrons(i, j);
					if(value > dblMaxIntensity) dblMaxIntensity = value;
				}
			}
			return dblMaxIntensity;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "computeMaxSourcesElectrons()", ex);}
		return Double.NaN;
	}
	
	
	
	
	


	public final void setNoiseElectrons(final int x, final int y, final int z, final double value)
	{
		getGeneratedNoiseAt(z).setNoiseElectrons(x, y, value);
	}
	
	public final double getNoiseElectrons(final int x, final int y, final int z)
	{
		return getGeneratedNoiseAt(z).getNoiseElectrons(x, y);
	}

	public final void validateNoise(final int z)
	{
		try{
			for(int x = 0; x < noises.length; x++){
				for(int y = 0; y < noises[0].length; y++){
					noises[x][y] += getGeneratedNoiseAt(z).getNoiseElectrons(x, y);
				}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "validateNoise()", ex);}
	}

	public final SResultNoise getGeneratedNoiseAt(final int i)
	{
		return (SResultNoise)(lstNoises.get(i));
	}
	
	public final int getGeneratedNoisesCount()
	{
		return lstNoises.size();
	}
	
	
	
	
	
	
	
	public final double getTotalElectrons(final int x, final int y)
	{
		return sources[x][y] + noises[x][y];
	}
	
	public final double computeMaxTotalElectrons()
	{
		try{
			double value = 0.0;
			double dblMaxIntensity = 0.0;
			for(int i = 0; i < sources.length; i++){
				for(int j = 0; j < sources[0].length; j++) {
					value = getTotalElectrons(i, j);
					if(value > dblMaxIntensity) dblMaxIntensity = value;
				}
			}
			return dblMaxIntensity;
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "computeMaxTotalElectrons()", ex);}
		return Double.NaN;
	}
	
	
	
	
	
	
	
	
	
	public final void setNormalizedValue(final int i, final int j, final int value)
	{
		if(value <= 0) norma[i][j] = -128;
		else if(value >= 254) norma[i][j] = 127;
		else norma[i][j] = (byte)( value - 128 );
	}

	public final int getNormalizedValue(final int i, final int j)
	{
		return norma[i][j] + 128;
	}
	

	
	
	
	

	
	public final int getRGBColorValue(final int i, final int j)
	{
		int val = getNormalizedValue(i, j);
		return ((255 & 0xFF) << 24) | ((val & 0xFF) << 16) | ((val & 0xFF) << 8) | ((val & 0xFF) << 0);
	}
	
	
	
	
	
	

	public final BufferedImage getNormalizedImage()
	{
		return image;
	}
	
	public final int getImageWidth()
	{
		return image.getWidth();
	}
	
	public final int getImageHeight()
	{
		return image.getHeight();
	}
	
	
	
	
	// le facteur utilise pour la conversion
	private double normalisationFactor			= Double.NaN;

	public final double getNormalisationFactor()
	{
		return normalisationFactor;
	}
	
	public final void setNormalisationFactor(final double normalisationFactor)
	{
		this.normalisationFactor = normalisationFactor;
	}
	
	
	
	
	
	// le facteur utilise pour la conversion en Jy/Ste
	private double toJyPerSteradFactor			= Double.NaN;

	public final double getToJyPerSteradFactor()
	{
		return toJyPerSteradFactor;
	}
	
	public final void setToJyPerSteradFactor(final double toJyPerSteradFactor)
	{
		this.toJyPerSteradFactor = toJyPerSteradFactor;
	}
	
	
	
	
	// le facteur utilise pour la conversion
	private double pixelResolution			= Double.NaN;
	
	public final double getPixelResolution()
	{
		return pixelResolution;
	}
	
	public final void setPixelResolution(final double pixelResolution)
	{
		this.pixelResolution = pixelResolution;
	}
	
	
}
