/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.framework.simulation.simul.specs.sources;

import sos.framework.simulation.simul.specs.SSpecImage;
import sos.lib.logger.Logger;

/**
 * Cette objet represente une source lumineuse abstraite.
 * 
 * @author Camille AMBROSI
 *
 */
public abstract class SSource
{

	/* position de la source */
	protected double			xCoord			= 0.0;
	protected double			yCoord			= 0.0;

	/* le flux total de la source en Jy */
	private double				dblTotalFlux	= 0.0;


	
	
	
	

	/**
	 * Constructeur generique d'une source.
	 * Une source est definie par sa position (coordonnees x et y), et par le flux qu'elle degage.
	 * 
	 * @param xCoord La position x de la source
	 * @param yCoord La position y de la source
	 * @param dblTotalFlux Le flux de la source, exprime en Jy
	 */
	protected SSource(final double xCoord, final double yCoord, final double dblTotalFlux)
	{
		try{
			/* position */
			this.xCoord = xCoord;
			this.yCoord = yCoord;
			/* flux */
			this.dblTotalFlux = dblTotalFlux;
		}catch (Exception ex) {Logger.fireError(this.getClass().getName(), "SOSSource()", ex);}
	}

	
	
	
	
	
	/**
	 * Retourne le flux total de la source, exprime en Jy.
	 * 
	 * @return Le flux total de la source, exprime en Jy.
	 */
	public final double getTotalFlux() {return dblTotalFlux;}
	
	/**
	 * Retourne la position x de la source dans la zone simulee, exprimee en pixel.
	 * 
	 * @return La position x de la source dans la zone simulee, exprimee en pixel.
	 */
	public final double getPosX() {return xCoord;}
	
	/**
	 * Retourne la position y de la source dans la zone simulee, exprimee en pixel.
	 * 
	 * @return La position y de la source dans la zone simulee, exprimee en pixel.
	 */
	public final double getPosY() {return yCoord;}

	
	
	

	/**
	 * Calcule le flux degage par la source dans le pixel situe au point de coordonnees (x,y), 
	 * en fonction de l'environnement de simulation.
	 * 
	 * @param xArcsec La coordonnee x du point
	 * @param yArcsec La coordonnee y du point
	 * @param image Les specifications de la simulation
	 * @return L'intensite calculee au point (x,y)
	 */
	public abstract double calculateIntensity(final double xArcsec, final double yArcsec, final SSpecImage image);

	
}



