/*
 SOS Project Classes.

 Copyright (C) 2006-2008 SOS Project <http://sos.project.free.fr>
 SOS Project is Denis BURGARELLA and Camille AMBROSI.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sos.lib.uicommon.graphics;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.text.DecimalFormat;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;

import sos.framework.maths.Constants;
import sos.framework.maths.Maths;
import sos.lib.logger.Logger;
import sos.lib.uicommon.UI;

public class SOSGraphCanvas extends JPanel implements ActionListener
{

	private static final long	serialVersionUID	= 1L;

	public static final Color	COLOR_BORDER		= new Color(20, 20, 20);
	public static final Color	COLOR_AXES			= new Color(200, 200, 200);
	public static final Color	COLOR_TEXT			= new Color(0, 0, 0);
	public static final Color	COLOR_BACK			= new Color(250, 250, 250);
	public static final Color[]	COLOR_FUNCS			= { new Color(250, 10, 10), new Color(10, 10, 255), new Color(10, 255, 10) };

	public static final int		SCALE_LINEAR		= 1;
	public static final int		SCALE_LOG			= 2;

	
	
	private Font				ftScale				= null;
	private Image				buffer				= null;
	
	private Dimension			dimAll				= null;
	private Dimension			dimZone				= new Dimension(0, 0);

	private Insets				insets				= new Insets(5, 28, 14, 22);

	private Vector				funcs				= new Vector();
	private Vector				lines1				= new Vector();
	private Vector				lines2				= new Vector();

	private double				xMin				= 0.0;
	private double				xMax				= 0.0;

	private double				xStep1				= 0.0;
	private double				xStep2				= 0.0;

	private boolean				blAutoY				= true;

	private double				yMin				= 0.0;
	private double				yMax				= 0.0;

	private double				yStep1				= 0.0;
	private double				yStep2				= 0.0;

	private int					iScaleX				= SCALE_LINEAR;
	private int					iScaleY				= SCALE_LINEAR;

	private double				dblPixelResX		= 0.0;
	private double				dblPixelResY		= 0.0;

	private JButton				btnX				= null;
	private JButton				btnY				= null;

	
	
	private String strXLabel						= null;
	private String strYLabel						= null;
	
	private String strXFormat						= null;
	private String strYFormat						= null;
	
	private boolean blIsHistogram					= false;
	
	
	
	
	
	
	

	
	

	public SOSGraphCanvas(final double xMin, final double xMax, final double xStep1, final double xStep2)
	{
		super(null);
		this.setOpaque(false);
		this.ftScale = new Font("Monospaced", Font.PLAIN, 9);
		btnX = UI.createButton("x", this, null);
		btnY = UI.createButton("y", this, null);
		//this.add(btnX);
		//this.add(btnY);
		this.setXPolicy(xMin, xMax, xStep1, xStep2);
		//this.setBorder(new LineBorder(Color.magenta));
	}



	
	
	
	public void paintComponent(Graphics g)
	{
		try{
			
			//btnX.setBounds(this.getWidth()-18, 5, 16, 16);
			//btnY.setBounds(this.getWidth()-18, 25, 16, 16);
			
			super.paintComponent(g);
			if((dimAll == null) || (dimAll.width != this.getWidth()) || (dimAll.height != this.getHeight()) || (buffer == null)){
				updateBufferImage();
			}
			g.drawImage(buffer, 0, 0, null);
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "paintComponent()", ex);}
	}




	public final void updateGraphic()
	{
		if(this.isVisible()){
			updateBufferImage();
			this.repaint();
		}
	}
	
	
	
	
	
	
	private final void updateBufferImage()
	{
		try{

			// demande les dimensions
			dimAll = this.getSize();
	
			dimZone.width = dimAll.width - insets.left - insets.right;
			dimZone.height = dimAll.height - insets.top - insets.bottom;	
			
			if(buffer != null){
				// TODO ne pas recreer toujours le buffer
			}
			buffer = null;

			// on regarde si la zone est valide
			if(dimZone.width > 0 && dimZone.height > 0){

				// cree le buffer
				buffer = this.createImage(dimAll.width, dimAll.height);
				Graphics g = buffer.getGraphics();
				g.setFont(ftScale);
	
				// on trace le fond
				paintBack(g);

				// facteur d'echelle en x
				if(iScaleX == SCALE_LINEAR) dblPixelResX = (xMax - xMin) / dimZone.width;
				else dblPixelResX = ( Maths.log(xMax) - Maths.log(xMin) ) / dimZone.width;

				// verifie qu'il y ait des fonctions
				if(funcs.size() == 0){
					paintError(g, "Aucune donnée");
					g.dispose();
					return;
				}

				// on calcule les points
				Vector[] data = new Vector[funcs.size()];
				for(int i = 0; i < funcs.size(); i++){
					data[i] = calculateValues((SFunction)(funcs.get(i)), i == 0);
				}

				// regarde si les bornes sont valides
				if( Double.isInfinite(yMin) || Double.isInfinite(yMax) || Double.isNaN(yMin) || Double.isNaN(yMax) || ( (iScaleY == SCALE_LOG) && (yMin <= 0) ) || ( (iScaleX == SCALE_LOG) && (xMin <= 0) ) ){
					paintError(g, "Impossible de tracer les bornes");
					g.dispose();
					return;
				}

				// facteur d'echelle en y
				if(iScaleY == SCALE_LINEAR){
					dblPixelResY = (yMax - yMin) / dimZone.height;
					// on calcule les graduations si besoin
					if(blAutoY == true){
						yStep1 = (yMax - yMin) / 5;
						yStep2 = (yMax - yMin) / 50;
					}
				}else{
					dblPixelResY = ( Maths.log(yMax) - Maths.log(yMin) ) / dimZone.height;
				}

				// on paint les graduations
				paintAxes(g, true, iScaleX, xMin, xMax, dimZone.width, xStep1, xStep2);
				paintAxes(g, false, iScaleY, yMin, yMax, dimZone.height, yStep1, yStep2);
				
				// on paint les axes en Y
				for(int i = 0; i < lines1.size(); i++){
					paintYAxis(g, ((Double)(lines1.get(i))).doubleValue());
				}
				
				// on paint les courbes
				for(int i = 0; i < data.length; i++){
					if(data[i] != null) paintCurve(g, data[i], getColor((SFunction)(funcs.get(i)), i));
				}

				// on paint les axes en Y
				for(int i = 0; i < lines2.size(); i++){
					paintYAxis(g, ((Double)(lines2.get(i))).doubleValue());
				}
				
				g.dispose();
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "updateBufferImage()", ex);}
	}

	
	
	
	/**
	 * Dessine le fond de l'image.
	 * 
	 * @param g
	 */
	private final void paintBack(final Graphics g)
	{
		try {
			// zone du graphique
			g.setColor(COLOR_BACK);
			g.fillRect(insets.left - 1, insets.top - 1, dimZone.width + 2, dimZone.height + 2);
			// tour de la zone du graphique
			g.setColor(COLOR_BORDER);
			g.drawRect(insets.left - 1, insets.top - 1, dimZone.width + 2, dimZone.height + 2);
			// labels
			if( strXLabel != null ){
				int fw = g.getFontMetrics().stringWidth(strXLabel);
				g.drawString(strXLabel, dimZone.width/2-fw/2+insets.left, dimAll.height-2);
				
			}
			if( (strYLabel != null) && (g instanceof Graphics2D) ){
				int fw = g.getFontMetrics().stringWidth(strYLabel);
				Graphics2D g2d = (Graphics2D)g;
				AffineTransform t = g2d.getTransform(); 
				g2d.rotate(-Constants.PI/2);
				g.drawString(strYLabel, -dimZone.height/2-insets.top-fw/2, 9);
				g2d.setTransform(t);
			}
		} catch (Exception ex) {Logger.fireError(this.getClass().getName(), "paintBack()", ex);}
	}
	
	
	
	

	/**
	 * Nottifie la presence d'une erreur par un message texte sur l'image.
	 * 
	 * @param g
	 * @param strMessage
	 */
	private final void paintError(final Graphics g, final String strMessage)
	{
		try {
			g.setColor(COLOR_TEXT);
			g.drawString(strMessage, insets.left + 6, insets.top + 14);
		} catch (Exception ex) {Logger.fireError(this.getClass().getName(), "paintError()", ex);}
	}
	
	
	
	
	
	/**
	 * Dessine les axes x ou y.
	 * 
	 * @param g
	 * @param blAxeX
	 * @param iScale
	 * @param min
	 * @param max
	 * @param pwidth
	 * @param step1
	 * @param step2
	 */
	private final void paintAxes(final Graphics g, final boolean blAxeX, final int iScale, final double min, final double max, final int pwidth, final double step1, final double step2)
	{
		try{
			int pxa1, pxa2;
			if(iScale == SCALE_LINEAR){
				// calcule les resolutions
				double pres = (max - min) / pwidth;
				double rp1 = step1 / pres;
				double rp2 = step2 / pres;
				// axes principaux
				if(rp1 >= 5){
					for(double px1 = 0; px1 < pwidth; px1 += rp1){
						pxa1 = (int)Math.round(px1);
						// ne dessine pas les axes trop prets des bords
						if(pxa1 < pwidth-1){
							drawString(g, blAxeX, getStr(min + px1 * pres, blAxeX), pxa1);
							if( pxa1 > 1 ) drawLine(g, blAxeX, pxa1, true, rp2 >= 3);
						}
						// axes secondaires
						if(rp2 >= 3){
							for(double px2 = rp2; (px2 < rp1) && (px2 < pwidth); px2 += rp2){
								pxa2 = pxa1 + (int)Math.round(px2);
								if( (pxa2 > 1) && (pxa2-pxa1 < rp1-1) && (pxa2 < pwidth-1) ) drawLine(g, blAxeX, pxa2, false, false);
							}
						}
					}
				}
				drawString(g, blAxeX, getStr(max, blAxeX), pwidth);				
			}else{
				// calcule le nombre de modules
				double nb = 0, d = min, mem = 0;
				while(d < max) {nb++; mem = d; d = d * 10;}
				// calcule la taille apres le dernier module
				if(d - max > 0) nb = nb - 1 + Maths.log(max / mem);
				double rp1 = pwidth / nb;
				int i = 0;
				// modules
				if(rp1 >= 5){
					for(double px1 = 0; px1 < pwidth; px1 += rp1){
						pxa1 = (int)Math.round(px1);
						// ne dessine pas les axes trop prets des bords
						if( pxa1 < pwidth-1 ){
							drawString(g, blAxeX, getStr(min * Math.pow(10, i), blAxeX), pxa1);
							if( pxa1 > 1 ) drawLine(g, blAxeX, pxa1, true, rp1 > 30);
						}
						// axes secondaires
						if( rp1 > 30 ){
							for(int j = 2; j < 10; j++){
								pxa2 = pxa1 + (int)(rp1 * Maths.log(j));
								if( pxa2 < pwidth-2 ) drawLine(g, blAxeX, pxa2, false, false);
							}
						}
						i++;
					}
				}
				drawString(g, blAxeX, getStr(max, blAxeX), pwidth);
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "paintAxes", ex);}
	}
	
	
	
	
	/**
	 * Dessine une chaine sur l'image le long de l'axe des x ou des y.
	 * 
	 * @param g
	 * @param blAxeX
	 * @param s
	 * @param index
	 */
	private void drawString(Graphics g, boolean blAxeX, String s, int index)
	{
		g.setColor(COLOR_BORDER);
		int fw = g.getFontMetrics().stringWidth(s);
		if(blAxeX == true){
			// centre la chaine
			g.drawString(s, index-fw/2+insets.left, insets.top+dimZone.height+13);
		}else{
			// aligne a droite
			g.drawString(s, insets.left-fw-1, insets.top+dimZone.height-index+(g.getFontMetrics().getHeight()-g.getFontMetrics().getDescent())/2);
		}
	}
	
	
	

	/**
	 * Dessine un axe sur l'image.
	 * 
	 * @param g
	 * @param blAxeX
	 * @param index
	 * @param major
	 */
	private void drawLine(Graphics g, boolean blAxeX, int index, boolean major, boolean width)
	{
		g.setColor(COLOR_AXES);
		if(blAxeX == true){
			// ligne grise complete
			g.drawLine(index+insets.left, insets.top, insets.left+index, insets.top+dimZone.height);
			if(major && width) g.drawLine(index+insets.left+1, insets.top, insets.left+index+1, insets.top+dimZone.height);
			// petite ligne noire
			g.setColor(COLOR_BORDER);
			g.drawLine(index+insets.left, insets.top+dimZone.height-(major?4:2), insets.left+index, insets.top+dimZone.height);
		}else{
			// ligne grise complete
			g.drawLine(insets.left, insets.top+dimZone.height-index, insets.left+dimZone.width, insets.top+dimZone.height-index);
			if(major && width) g.drawLine(insets.left, insets.top+dimZone.height-index-1, insets.left+dimZone.width, insets.top+dimZone.height-index-1);
			// petite ligne noire
			g.setColor(COLOR_BORDER);
			g.drawLine(insets.left, insets.top+dimZone.height-index, insets.left+(major?4:2), insets.top+dimZone.height-index);
		}
	}

	
	
	
	/**
	 * Retourne une valeur arrondie pour un double.
	 * 
	 * @param in
	 * @return
	 */
	private String getStr(double in, boolean blAxeX)
	{
		String format = "0.#E0";
		if(  blAxeX && (strXFormat != null) ) format = strXFormat;
		if( !blAxeX && (strYFormat != null) ) format = strYFormat;
		String str = new DecimalFormat(format).format(in).replace(',', '.').replace('E', 'e');
		if(str.endsWith("e0")) return str.substring(0, str.length() - 2);
		return str;
	}




	
	
	



	

	private final void paintYAxis(final Graphics g, final double xReal)
	{
		try{
			// couleur de cette fonction
			g.setColor(Color.black);

			if(Double.isNaN(xReal) == false && Double.isInfinite(xReal) == false ){
				// calcule la valeur du pixel en x dépendant de l'echelle
				int x;
				
				if(iScaleY == SCALE_LINEAR) x = (int)Math.round( ( xReal - xMin ) / dblPixelResX );
				else x = (int)Math.round( ( Maths.log(xReal) - Maths.log(xMin) ) / dblPixelResX );
				
				g.drawLine(insets.left + x, insets.top, insets.left + x, insets.top + dimZone.height);
		
			}
			
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "paintCurve()", ex);}
	}
	
	
	
	
	/**
	 * Dessine la fonction sur le graphique.
	 * 
	 * @param g
	 * @param data
	 * @param color
	 */
	private final void paintCurve(final Graphics g, final Vector data, final Color color)
	{
		try{
			int y;
			int yMem = 0;
			double dblRealY = 0.0;
			boolean yMemSet = false;
			// couleur de cette fonction
			g.setColor(color);
			// parcours tous les pixels en x
			for(int x = 0; x <= dimZone.width; x++){
				// valeur en x de la fonction
				dblRealY = ((Double)(data.get(x))).doubleValue();
				// verifie que c'est bien une valeur
				if(Double.isNaN(dblRealY) == false && Double.isInfinite(dblRealY) == false ){
					// calcule la valeur du pixel en x dépendant de l'echelle
					if(iScaleY == SCALE_LINEAR) y = (int)Math.round( ( dblRealY - yMin ) / dblPixelResY );
					else y = (int)Math.round( ( Maths.log(dblRealY) - Maths.log(yMin) ) / dblPixelResY );
					// regarde si le point ne sort pas de la zone
					if( (dblRealY >= yMin) && (dblRealY <= yMax) ){
						if(blIsHistogram == false){
							// regarde si le point précédent est valide
							if(yMemSet){
								if(yMem < 0) yMem=0;
								if(yMem > dimZone.height) yMem=dimZone.height;
								g.drawLine(insets.left + x - 1, insets.top + dimZone.height - yMem, insets.left + x, insets.top + dimZone.height - y);
							}
						}else{
							g.drawLine(insets.left + x, insets.top + dimZone.height, insets.left + x, insets.top + dimZone.height - y);
						}
					}
					yMem = y;
					yMemSet = true;
				}else yMemSet = false;
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "paintCurve()", ex);}
	}
	
	
	
	
	/**
	 * Calcule les valeurs de la fonction passee en parametre pour chaque pixel et  
	 * retourne le resultat dans un vecteur.
	 * 
	 * @param func
	 * @param blFirstFunction
	 * @return
	 */
	private final Vector calculateValues(final SFunction func, final boolean blFirstFunction)
	{
		Vector data = new Vector(dimZone.width);
		try{
			double dblRealX = 0.0;
			double dblRealY = 0.0;
			for(int x = 0; x <= dimZone.width; x++){
				// calcule la valeur en x a partir de la position du pixel
				if(iScaleX == SCALE_LINEAR) dblRealX = xMin + x * dblPixelResX;
				else dblRealX = Maths.pow( 10.0, x * dblPixelResX + Maths.log( xMin ) );
				// calcule la valeur en y
				dblRealY = func.yValue( dblRealX );
				data.add( new Double( dblRealY ) );
				// regarde s'il faut déterminer les bornes
				if( blAutoY ){
					if( (x == 0) && (blFirstFunction == true) ){
						yMin = dblRealY;
						yMax = dblRealY;
					}else{
						if(dblRealY < yMin || Double.isNaN(yMin)) yMin = dblRealY;
						if(dblRealY > yMax || Double.isNaN(yMax)) yMax = dblRealY;
					}
				}
			}
		}catch(Exception ex) {Logger.fireError(this.getClass().getName(), "calculateValues()", ex);}
		return data;
	}

	
	
	
	
	/**
	 * Retourne une couleur pour dessiner la fonction. Si la fonction n'a pas de couleur, 
	 * alors retourne une couleur courante.
	 * 
	 * @param func
	 * @param index
	 * @return
	 */
	private final Color getColor(final SFunction func, final int index)
	{
		if(func.getColor() != null) return func.getColor();
		else return COLOR_FUNCS[index % COLOR_FUNCS.length];
	}
	
	
	
	

	
	

	

	
	public void actionPerformed(ActionEvent ae)
	{
		if(ae.getSource() == btnX){
			if(iScaleX == SCALE_LINEAR) setXScale(SCALE_LOG);
			else setXScale(SCALE_LINEAR);
			updateGraphic();
		}
		if(ae.getSource() == btnY){
			if(iScaleY == SCALE_LINEAR) setYScale(SCALE_LOG);
			else setYScale(SCALE_LINEAR);
			updateGraphic();
		}	
	}
	

	
	
	
	
	
	
	
	

	public final void setXPolicy(final double xMin, final double xMax, final double xStep1, final double xStep2)
	{
		this.xMin = xMin;
		this.xMax = xMax;
		this.xStep1 = xStep1;
		this.xStep2 = xStep2;
	}

	public final void setYPolicy(final double yMin, final double yMax, final double yStep1, final double yStep2)
	{
		this.yMin = yMin;
		this.yMax = yMax;
		this.yStep1 = yStep1;
		this.yStep2 = yStep2;
		this.blAutoY = false;
	}
	
	public final void setYPolicyAuto() {blAutoY = true;}

	public final void setXLabel(final String strXLabel)
	{
		this.strXLabel = strXLabel;
		if( strXLabel != null ) insets.bottom = 24;
		else insets.bottom = 14;
	}
	
	public final void setYLabel(final String strYLabel)
	{
		this.strYLabel = strYLabel;
		if( strYLabel != null ) insets.left = 38;
		else insets.left = 28;
	}
	
	public final void clearFunctions() {funcs.clear();}
	public final void addFunction(final SFunction func) {funcs.add(func);}
	
	public final void addYAxisBack(final double yvalue) {lines1.add(new Double(yvalue));}
	public final void clearYAxisBack() {lines1.clear();}
	
	public final void addFrontYAxis(final double yvalue) {lines2.add(new Double(yvalue));}
	public final void clearFrontYAxis() {lines2.clear();}

	public final void setXScale(final int iScaleX) {this.iScaleX = iScaleX;}
	public final void setYScale(final int iScaleY) {this.iScaleY = iScaleY;}
	
	public final void setXFormat(final String strXFormat) {this.strXFormat = strXFormat;}
	public final void setYFormat(final String strYFormat) {this.strYFormat = strYFormat;}

	public final void setHistogram(final boolean blIsHistogram) {this.blIsHistogram = blIsHistogram;}
	

}
